module Util exposing (..)
--Http
import Http

maybeIntToMaybeFloat : Maybe Int -> Maybe Float
maybeIntToMaybeFloat mayInt =
    case mayInt of
        Just int -> Just (toFloat int)
        Nothing  -> Nothing

-- Working with Lists

list_Last : List a -> Maybe a
list_Last list =
  case list of
    []    -> Nothing
    x::[] -> Just x
    x::xs -> list_Last xs

list_next : List a -> List a
list_next list = List.append (List.drop 1 list) (List.take 1 list)

list_swap : List a -> {i1: Int,i2: Int} -> List a -- swaps the 2 elements at that 2 indices in the input list
list_swap list {i1,i2} =
    let e1 : a
        e1 = list_get list i1
        e2 : a
        e2 = list_get list i2
    in list_set (list_set list i1 e2) i2 e1

list_get : List a -> Int -> a -- returns the listelement at index i. creates endless loop if length list < i+1
list_get list i =
    case list of
        []    -> list_get list i
        x::xs -> if i == 0 then x else list_get xs (i-1)

list_set : List a -> Int -> a -> List a -- resets at index i the element with e. does nothing if length list < i+1
list_set list i e =
    case list of
        []    -> []
        x::xs -> if i==0 then e::xs else x::(list_set xs (i-1) e)

list_remove : List a -> a -> List a
list_remove list e = List.filter (\x -> x/=e) list

list_groupBy : (a -> comparable) -> List a -> List (List a) -- groups the list by the comparator. returns empty list if input is empty.
list_groupBy comparator list =
  let sortedList = List.sortBy comparator list

      groupByHelper : (a -> comparable) -> List a -> List a -> a -> List (List a) -> List (List a)
      groupByHelper c inList lastGroup lastValue out =
        case inList of
          []    -> lastGroup::out
          x::xs -> if c x == c lastValue
                   then groupByHelper c xs (x::lastGroup) x out
                   else groupByHelper c xs [x] x (lastGroup::out)
  in case sortedList of
       []    -> []
       x::xs -> groupByHelper comparator xs [x] x []

list_maximumBy : (a -> comparable) -> List a -> a
list_maximumBy comparator list =
  case list of
    [] -> list_maximumBy comparator list -- endless loop if input is empty
    x::xs -> List.foldl (\current last ->
               if comparator current > comparator last then current else last
             ) x xs

list_getIndexOf : a -> List a -> Int -- returns the index of the 1st occurence of the specified element in the input list
list_getIndexOf global_e global_list =
  let getIndexOfHelper : a -> List a -> Int -> Int
      getIndexOfHelper e list i =
        case list of
          []    -> -1
          x::xs -> if e == x then i else getIndexOfHelper e xs (i+1)
  in getIndexOfHelper global_e global_list 0


-- Working with Errors
httpErrorToString : Http.Error -> String
httpErrorToString err =
  "Http-Error: "++(
    case err of
      Http.BadUrl string  -> "BadUrl "++string
      Http.Timeout        -> "Timeout"
      Http.NetworkError   -> "NetworkError"
      Http.BadStatus int  -> "BadStatus "++(String.fromInt int)
      Http.BadBody string -> "BadBody "++string
  )